/*
    Resize Active Comp
        あかつきみさき

    このスクリプトについて
        アクティブなコンポジションの幅・高さを下位コンポ層を含めて一括で変更するスクリプト.

        Getボタンでアクティブなアイテムの幅・高さを取得します.
        Setボタンでアクティブなコンポジションに幅・高さを設定します.

    使用方法
        ファイル→スクリプト→スクリプトファイルを実行.
        「ScriptUI Panels」フォルダーに入れて実行

    動作環境
        Adobe After Effects CS6以上

    バージョン情報
        2019/05/06 Ver 1.0.0 Release
*/

/// <reference path="C:\Users\RUI\OneDrive\lib\aftereffects.d.ts\ae.d.ts"/>

(function () {
    var REPOSITION_CENTER = 3819;
    var FIT_TO_COMP = 2156;

    CompItem.prototype.resizeWithChildrenComp = function (width, height, repositionCenter, fitToComp, applyId) {
        for (var i = 0; i < applyId.length; i++) {
            if (applyId[i] == this.id) {
                return 0;
            }
        }

        applyId[applyId.length] = this.id;

        for (var j = 1; j <= this.layers.length; j++) {
            if (this.layer(j).source instanceof CompItem) {
                this.layer(j).source.resizeWithChildrenComp(width, height, repositionCenter, fitToComp, applyId);
            }
        }

        this.width = width;
        this.height = height;

        if (repositionCenter) {
            this.openInViewer();

            for (var k = 1; k <= this.layers.length; k++) {
                this.layer(k).selected = true;
                app.executeCommand(REPOSITION_CENTER);
                this.layer(k).selected = false;
            }
        }

        if (fitToComp) {
            this.openInViewer();

            for (var k = 1; k <= this.layers.length; k++) {
                this.layer(k).selected = true;
                app.executeCommand(FIT_TO_COMP);
                this.layer(k).selected = false;
            }
        }

        return 0;
    }

    function buildUI(thisObj) {
        if (thisObj instanceof Panel) {
            var resizeWindow = thisObj;
        } else {
            var resizeWindow = new Window("palette", "Resize Active Comp", undefined, { resizeable: true });
        }

        resizeWindow.orientation = "column";
        resizeWindow.alignment = ["fill", "top"];
        resizeWindow.margins = [1, 1, 1, 1];
        resizeWindow.spacing = 0;

        var compGrp = resizeWindow.add("group");
        compGrp.margins = [1, 1, 1, 1];
        compGrp.spacing = 0;

        compGrp.alignment = ["fill", "top"];
        compGrp.orientation = "column";

        var widthGrp = compGrp.add("group");
        widthGrp.margins = [1, 1, 1, 1];
        widthGrp.spacing = 0;

        widthGrp.alignment = ["fill", "top"];
        widthGrp.orientation = "row";

        var widthCb = widthGrp.add("checkbox");
        widthCb.alignment = ["fill", "top"];
        widthCb.value = true;
        widthCb.text = "width";

        var widthEt = widthGrp.add("edittext");
        widthEt.alignment = ["fill", "top"];
        widthEt.text = "1920";
        widthEt.characters = 10;

        var heightGrp = compGrp.add("group");
        heightGrp.margins = [1, 1, 1, 1];
        heightGrp.spacing = 0;

        heightGrp.alignment = ["fill", "top"];
        heightGrp.orientation = "row";

        var heightCb = heightGrp.add("checkbox");
        heightCb.alignment = ["fill", "top"];
        heightCb.value = true;
        heightCb.text = "height";

        var heightEt = heightGrp.add("edittext");
        heightEt.alignment = ["fill", "top"];
        heightEt.text = "1080";
        heightEt.characters = 10;

        var optionGrp = resizeWindow.add("group");
        optionGrp.alignment = ["fill", "top"];
        optionGrp.orientation = "row";

        var repositionCb = optionGrp.add("checkbox");
        repositionCb.alignment = ["fill", "top"];
        repositionCb.value = true;
        repositionCb.text = "Reposition center";

        var fitCb = optionGrp.add("checkbox");
        fitCb.alignment = ["fill", "top"];
        fitCb.value = true;
        fitCb.text = "Fit to comp";

        var btnGrp = resizeWindow.add("group");
        btnGrp.alignment = ["fill", "top"];
        btnGrp.orientation = "row";

        var getBtn = btnGrp.add("button", undefined, "Get");
        getBtn.alignment = ["fill", "top"];

        var setBtn = btnGrp.add("button", undefined, "Set");
        setBtn.alignment = ["fill", "top"];

        widthEt.onChange = function () {
            if (isNaN(this.text) || this.text == "" || this.text < 0) {
                this.text = 1920;
            }
        }

        heightEt.onChange = function () {
            if (isNaN(this.text) || this.text == "" || this.text < 0) {
                this.text = 1080;
            }
        }

        widthCb.onClick = function () {
            widthEt.enabled = this.value;
        }

        heightCb.onClick = function () {
            heightEt.enabled = this.value;
        }


        getBtn.onClick = function () {
            var actItem = app.project.activeItem;
            if (actItem == null) {
                return;
            }

            if (widthCb.value) {
                widthEt.text = actItem.width;
            }

            if (heightCb.value) {
                heightEt.text = actItem.height;
            }
        }

        setBtn.onClick = function () {
            var actComp = app.project.activeItem;
            var applyId = [];

            if (actComp == null || !(actComp instanceof CompItem)) {
                return;
            }

            var width = actComp.width;
            var height = actComp.height;

            if (widthCb.value) {
                width = widthEt.text;
            }

            if (heightCb.value) {
                height = heightEt.text;
            }

            app.beginUndoGroup("Resize Active Comp");
            actComp.resizeWithChildrenComp(Number(width), Number(height), repositionCb.value, fitCb.value, applyId);
            app.endUndoGroup();
        }

        resizeWindow.layout.layout();
        resizeWindow.onResizing = resizeWindow.onResize = function () {
            this.layout.resize();
        }

        if (!(thisObj instanceof Panel)) {
            resizeWindow.center();
            resizeWindow.show();
        }

        return 0;
    }

    buildUI(this);

    return 0;
}).call(this);

